--[[-------------------------------------------------------------------------
	SmallBlizRaidFrames
	Copyright (C) 2010  Morsker
	Please contact Morsker through PM on forums.wowace.com.

	Resizes Blizzard raid frames to be smaller squares.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
---------------------------------------------------------------------------]]
local addonName = ...
local SBRF = LibStub("AceAddon-3.0"):NewAddon(addonName)
_G.SmallBlizRaidFrames = SBRF

--[[ testing macro:
/run CompactPartyFrame:Show();CompactPartyFrame:ClearAllPoints();CompactPartyFrame:SetPoint("LEFT")
/run frame = CompactPartyFrameMember1
--]]

--------------
--  Locals  --
--------------

-- GLOBALS: _G
-- GLOBALS: SmallBlizRaidFramesDB
-- GLOBALS: EnableAddOn
-- GLOBALS: LoadAddOn
-- GLOBALS: LibStub
-- GLOBALS: CreateFrame
-- GLOBALS: InterfaceOptionsFrame_OpenToCategory
-- GLOBALS: InterfaceOptions_AddCategory
-- GLOBALS: InterfaceOptionsRaidFramePanelRaidFrameHeightSlider
-- GLOBALS: InterfaceOptionsRaidFramePanelRaidFrameWidthSlider
-- GLOBALS: CompactPartyFrame
-- GLOBALS: CompactRaidFrameContainer
-- GLOBALS: CompactRaidFrameManagerContainerResizeFrame
-- GLOBALS: CompactRaidFrameManager_ResetContainerPosition
-- GLOBALS: GetCVar
-- GLOBALS: SetCVar

-- GLOBALS: pairs
-- GLOBALS: unpack

local GameFontHighlightLarge = GameFontHighlightLarge
local GetScreenHeight = GetScreenHeight
local hooksecurefunc = hooksecurefunc
local bliz_options = DefaultCompactUnitFrameSetupOptions
local Texture_SetSize = ActionButton1Icon.SetSize

local DBVERSION = 1
local db
local fixedFrames = {}
local anchorData = {
	["TOPLEFT"]     = {  3, -2,  "LEFT", "RIGHT" },
	["TOPRIGHT"]    = { -3, -2, "RIGHT",  "LEFT" },
	["BOTTOMLEFT"]  = {  3,  2,  "LEFT", "RIGHT" },
	["BOTTOMRIGHT"] = { -3,  2, "RIGHT",  "LEFT" },
}

SBRF.fixedFrames = fixedFrames
SBRF.anchorData = anchorData

local centerNames
local verticalHP
local alpha
local showBuffs
local showDebuffs
local showDispelDebuffs
local buffAnchor
local debuffAnchor
local d_debuffAnchor
local     buffAnchor_x,     buffAnchor_y,     buffAnchor_point,     buffAnchor_relative
local   debuffAnchor_x,   debuffAnchor_y,   debuffAnchor_point,   debuffAnchor_relative
local d_debuffAnchor_x, d_debuffAnchor_y, d_debuffAnchor_point, d_debuffAnchor_relative
function SBRF:PushUpvalues()
	centerNames = db.centerNames
	verticalHP = db.verticalHP
	alpha = db.alpha

	showBuffs = db.showBuffs
	showDebuffs = db.showDebuffs
	showDispelDebuffs = db.showDispelDebuffs

	buffAnchor = db.buffAnchor
	debuffAnchor = db.debuffAnchor
	d_debuffAnchor = db.d_debuffAnchor

	    buffAnchor_x,     buffAnchor_y,     buffAnchor_point,     buffAnchor_relative = unpack(anchorData[buffAnchor])
	  debuffAnchor_x,   debuffAnchor_y,   debuffAnchor_point,   debuffAnchor_relative = unpack(anchorData[debuffAnchor])
	d_debuffAnchor_x, d_debuffAnchor_y, d_debuffAnchor_point, d_debuffAnchor_relative = unpack(anchorData[d_debuffAnchor])
end

--((@)) local counts = {
--((@)) 	fixframe = 0,
--((@)) 	hook2 = 0,
--((@)) 	name = 0,
--((@)) 	force32 = 0,
--((@)) }
--((@)) _G.SBRF = SBRF
--((@)) SBRF.counts = counts

---------------------
--  Init / Events  --
---------------------

local function HideMe(self)
	self:Hide()
end

SBRF.HideMe = HideMe

function SBRF.KeepDebuff32_32(debuffFrame)
	--((@)) counts.force32 = counts.force32 + 1
	Texture_SetSize(debuffFrame, 32, 32)
end

function SBRF.FixFrame(frame)
	--((@)) counts.fixframe = counts.fixframe + 1

	local name = frame.name
	name:SetFontObject(GameFontHighlightLarge)
	if centerNames then
		name:ClearAllPoints()
		name:SetPoint("CENTER")
	end

	Texture_SetSize(frame.roleIcon, 24, 24)

	if verticalHP then
		local healthBar = frame.healthBar
		local myHealPredictionBar = frame.myHealPredictionBar
		local otherHealPredictionBar = frame.otherHealPredictionBar
		myHealPredictionBar:ClearAllPoints()
		myHealPredictionBar:SetPoint("BOTTOMLEFT", healthBar:GetStatusBarTexture(), "TOPLEFT", 0, 0);
		myHealPredictionBar:SetPoint("BOTTOMRIGHT", healthBar:GetStatusBarTexture(), "TOPRIGHT", 0, 0);
		myHealPredictionBar:SetHeight(bliz_options.height)
		otherHealPredictionBar:ClearAllPoints()
		otherHealPredictionBar:SetPoint("BOTTOMLEFT", myHealPredictionBar:GetStatusBarTexture(), "TOPLEFT", 0, 0);
		otherHealPredictionBar:SetPoint("BOTTOMRIGHT", myHealPredictionBar:GetStatusBarTexture(), "TOPRIGHT", 0, 0);
		otherHealPredictionBar:SetHeight(bliz_options.height)
	end

	if showBuffs then
		local buffs = frame.buffFrames
		Texture_SetSize(buffs[1], 32, 32)
		Texture_SetSize(buffs[2], 32, 32)
		buffs[1]:ClearAllPoints()
		buffs[1]:SetPoint(buffAnchor, frame, buffAnchor, buffAnchor_x, buffAnchor_y)
		if buffAnchor_point ~= "RIGHT" then
			buffs[2]:ClearAllPoints()
			buffs[2]:SetPoint(buffAnchor_point, buffs[1], buffAnchor_relative, 0, 0)
		end
	end
	if showDebuffs then
		local debuffs = frame.debuffFrames
		debuffs[1]:ClearAllPoints()
		debuffs[1]:SetPoint(debuffAnchor, frame, debuffAnchor, debuffAnchor_x, debuffAnchor_y)
		if debuffAnchor_point ~= "LEFT" then
			debuffs[2]:ClearAllPoints()
			debuffs[2]:SetPoint(debuffAnchor_point, debuffs[1], debuffAnchor_relative, 0, 0)
		end
	end
	if showDispelDebuffs then
		local d_debuffs = frame.dispelDebuffFrames
		Texture_SetSize(d_debuffs[1], 32, 32)
		Texture_SetSize(d_debuffs[2], 32, 32)
		if d_debuffAnchor_point ~= "RIGHT" then
			d_debuffs[2]:ClearAllPoints()
			d_debuffs[2]:SetPoint(d_debuffAnchor_point, d_debuffs[1], d_debuffAnchor_relative, 0, 0)
		end
	end

	if not fixedFrames[frame] then
		fixedFrames[frame] = true
		local buffs = frame.buffFrames
		local debuffs = frame.debuffFrames
		local d_debuffs = frame.dispelDebuffFrames

		-- make debuffs 32x32
		hooksecurefunc(debuffs[1], "SetSize", SBRF.KeepDebuff32_32)
		hooksecurefunc(debuffs[2], "SetSize", SBRF.KeepDebuff32_32)

		-- vertical hp bars
		if verticalHP then
			frame.healthBar:SetOrientation("VERTICAL")
			frame.healthBar:SetRotatesTexture(true)
			frame.myHealPredictionBar:SetOrientation("VERTICAL")
			frame.myHealPredictionBar:SetRotatesTexture(true)
			frame.otherHealPredictionBar:SetOrientation("VERTICAL")
			frame.otherHealPredictionBar:SetRotatesTexture(true)
			-- frame.powerBar ? Making this vertical is more work, I'll see what people want first.
		end
		
		-- hide stuff we want hidden
		if not showBuffs then
			buffs[1]:Hide()
			buffs[1]:SetScript("OnShow", SBRF.HideMe)
			buffs[2]:Hide()
			buffs[2]:SetScript("OnShow", SBRF.HideMe)
		end
		if not showDebuffs then
			debuffs[1]:Hide()
			debuffs[1]:SetScript("OnShow", SBRF.HideMe)
			debuffs[2]:Hide()
			debuffs[2]:SetScript("OnShow", SBRF.HideMe)
		end
		if not showDispelDebuffs then
			d_debuffs[1]:Hide()
			d_debuffs[1]:SetScript("OnShow", SBRF.HideMe)
			d_debuffs[2]:Hide()
			d_debuffs[2]:SetScript("OnShow", SBRF.HideMe)
		end

		-- hide 3rd buffs
		buffs[3]:Hide()
		buffs[3]:SetScript("OnShow", HideMe)
		debuffs[3]:Hide()
		debuffs[3]:SetScript("OnShow", HideMe)
		d_debuffs[3]:Hide()
		d_debuffs[3]:SetScript("OnShow", HideMe)
	end
end

hooksecurefunc("DefaultCompactUnitFrameSetup", SBRF.FixFrame)
hooksecurefunc("DefaultCompactMiniFrameSetup", SBRF.FixFrame)
hooksecurefunc("CompactRaidFrameReservation_RegisterReservation", function(_, frame, _)
	--((@)) counts.hook2 = counts.hook2 + 1
	SBRF.FixFrame(frame)
end)

function SBRF.TruncateFrameName(frame)
	--((@)) counts.name = counts.name + 1
	local name = frame.name
	local nameText = name:GetText()
	if #nameText > 5 then
		name:SetText(nameText:sub(1,5))
	end
end

hooksecurefunc("CompactUnitFrame_UpdateName", SBRF.TruncateFrameName)

function SBRF.FixResizeFrame(frame)
	local resizeFrame = frame.containerResizeFrame
	resizeFrame:SetMaxResize(resizeFrame:GetWidth(), (GetScreenHeight()*2) - 90)
end

hooksecurefunc("CompactRaidFrameManager_UpdateContainerBounds", SBRF.FixResizeFrame)

function SBRF.FixMagnetism(manager)
	local point, relativeTo, relativePoint, xOffset, yOffset = manager.containerResizeFrame:GetPoint(1)
	if relativeTo == manager then
		CompactRaidFrameManager_ResetContainerPosition()
	end
end

hooksecurefunc("CompactRaidFrameManager_ResizeFrame_CheckMagnetism", SBRF.FixMagnetism)

local function LoadOptions()
	if SBRF.options_table then return end
	EnableAddOn("SmallBlizRaidFrames_Options")
	LoadAddOn("SmallBlizRaidFrames_Options")
	SBRF:InitOptions()
end

-- GLOBALS: SLASH_SMALLBLIZRAIDFRAMES1
-- GLOBALS: SLASH_SMALLBLIZRAIDFRAMES2
SLASH_SMALLBLIZRAIDFRAMES1 = "/sbrf"
SLASH_SMALLBLIZRAIDFRAMES2 = "/smallbrf"
function SlashCmdList.SMALLBLIZRAIDFRAMES()
	LoadOptions()
	LibStub("AceConfigDialog-3.0"):Open("SmallBlizRaidFrames")
end

function SBRF:OnInitialize()
	self.OnInitialize = nil
	SmallBlizRaidFramesDB = SmallBlizRaidFramesDB or { dbversion = DBVERSION }
	db = SmallBlizRaidFramesDB
	SBRF.db = db
	local defaults = {
		centerNames = true,
		verticalHP = true,
		alpha = 1.0,
		buffAnchor = "BOTTOMRIGHT",
		debuffAnchor = "BOTTOMLEFT",
		d_debuffAnchor = "TOPRIGHT",
		showBuffs = true,
		showDebuffs = true,
		showDispelDebuffs = true,
	}
	for k,v in pairs(defaults) do
		if db[k] == nil then
			db[k] = defaults[k]
		end
	end
	self:PushUpvalues()

	-- UI stub for opening options, but only if we need it.
	if not SBRF.options_table then
		local dummyOptionsFrame = CreateFrame("Frame")
		dummyOptionsFrame.name = "SmallBlizRaidFrames"
		dummyOptionsFrame:Hide()
		dummyOptionsFrame:SetScript("OnShow", function(frame)
			frame:SetScript("OnShow", nil)
			frame:Hide()
			LoadOptions()
			InterfaceOptionsFrame_OpenToCategory("SmallBlizRaidFrames")
		end)
		InterfaceOptions_AddCategory(dummyOptionsFrame, "SmallBlizRaidFrames")
	end

	CompactRaidFrameContainer:SetScale(0.5)
	CompactRaidFrameManagerContainerResizeFrame:SetScale(0.5)
	CompactPartyFrame:SetScale(0.7)

	CompactRaidFrameContainer:SetAlpha(alpha)
	CompactPartyFrame:SetAlpha(alpha)
end

function SBRF:OnEnable()
end

function SBRF:OnDisable()
end
