--[[-------------------------------------------------------------------------
	SmallBlizRaidFrames
	Copyright (C) 2010  Morsker
	Please contact Morsker through PM on forums.wowace.com.

	Resizes Blizzard raid frames to be smaller squares.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
---------------------------------------------------------------------------]]

local SBRF = SmallBlizRaidFrames
local db = SBRF.db
local fixedFrames = SBRF.fixedFrames
local anchorData = SBRF.anchorData
local bliz_options = DefaultCompactUnitFrameSetupOptions

-- GLOBALS: LibStub
-- GLOBALS: CompactPartyFrame
-- GLOBALS: CompactRaidFrameContainer
-- GLOBALS: CompactUnitFrame_UpdateAuras
-- GLOBALS: INTERFACEOPTIONS_ADDONCATEGORIES
-- GLOBALS: InterfaceAddOnsList_Update
-- GLOBALS: SetCVar
-- GLOBALS: GetCVarBool
-- GLOBALS: tremove
-- GLOBALS: pairs
-- GLOBALS: unpack

local Texture_SetSize = ActionButton1Icon.SetSize

local anchor_values = {
	["TOPLEFT"] = "TopLeft",
	["TOPRIGHT"] = "TopRight",
	["BOTTOMLEFT"] = "BottomLeft",
	["BOTTOMRIGHT"] = "BottomRight",
}
local buffFrameMap = {
	showBuffs = "buffFrames",
	buffAnchor = "buffFrames",
	showDebuffs = "debuffFrames",
	debuffAnchor = "debuffFrames",
	showDispelDebuffs = "dispelDebuffFrames",
	d_debuffAnchor = "dispelDebuffFrames",
}
local anchorShowMap = {
	buffAnchor = "showBuffs",
	debuffAnchor = "showDebuffs",
	d_debuffAnchor = "showDispelDebuffs",
}

local function SetCenterNames(info, val)
	db.centerNames = val
	SBRF:PushUpvalues()
	for frame,_ in pairs(fixedFrames) do
		local name = frame.name
		name:ClearAllPoints()
		if val then
			name:SetPoint("CENTER")
		else
			name:SetPoint("TOPLEFT", frame.roleIcon, "TOPRIGHT", 0, -1);
			name:SetPoint("TOPRIGHT", -3, -3);
		end
	end
end

local function SetVerticalHP(info, val)
	db.verticalHP = val
	SBRF:PushUpvalues()
	local orientation = val and "VERTICAL" or "HORIZONTAL"
	for frame,_ in pairs(fixedFrames) do
		local healthBar = frame.healthBar
		local myHealPredictionBar = frame.myHealPredictionBar
		local otherHealPredictionBar = frame.otherHealPredictionBar
		healthBar:SetOrientation(orientation)
		myHealPredictionBar:SetOrientation(orientation)
		otherHealPredictionBar:SetOrientation(orientation)
		myHealPredictionBar:ClearAllPoints()
		otherHealPredictionBar:ClearAllPoints()
		if val then
			myHealPredictionBar:SetPoint("BOTTOMLEFT", healthBar:GetStatusBarTexture(), "TOPLEFT", 0, 0);
			myHealPredictionBar:SetPoint("BOTTOMRIGHT", healthBar:GetStatusBarTexture(), "TOPRIGHT", 0, 0);
			myHealPredictionBar:SetHeight(bliz_options.height)
			otherHealPredictionBar:SetPoint("BOTTOMLEFT", myHealPredictionBar:GetStatusBarTexture(), "TOPLEFT", 0, 0);
			otherHealPredictionBar:SetPoint("BOTTOMRIGHT", myHealPredictionBar:GetStatusBarTexture(), "TOPRIGHT", 0, 0);
			otherHealPredictionBar:SetHeight(bliz_options.height)
		else
			myHealPredictionBar:SetPoint("TOPLEFT", healthBar:GetStatusBarTexture(), "TOPRIGHT", 0, 0);
			myHealPredictionBar:SetPoint("BOTTOMLEFT", healthBar:GetStatusBarTexture(), "BOTTOMRIGHT", 0, 0);
			myHealPredictionBar:SetWidth(bliz_options.width);
			otherHealPredictionBar:SetPoint("TOPLEFT", myHealPredictionBar:GetStatusBarTexture(), "TOPRIGHT", 0, 0);
			otherHealPredictionBar:SetPoint("BOTTOMLEFT", myHealPredictionBar:GetStatusBarTexture(), "BOTTOMRIGHT", 0, 0);
			otherHealPredictionBar:SetWidth(bliz_options.width);
		end
	end
end

local function SetAlpha(info, val)
	db.alpha = val
	SBRF:PushUpvalues()
	CompactRaidFrameContainer:SetAlpha(val)
	CompactPartyFrame:SetAlpha(val)
end

local function SetShowBuffs(info, val)
	local setting = info[#info]
	db[setting] = val
	SBRF:PushUpvalues()
	local frameType = buffFrameMap[setting]
	for frame,_ in pairs(fixedFrames) do
		local buffs = frame[frameType]
		if val then
			buffs[1]:SetScript("OnShow", nil)
			Texture_SetSize(buffs[1], 32, 32)
			buffs[2]:SetScript("OnShow", nil)
			Texture_SetSize(buffs[2], 32, 32)
			CompactUnitFrame_UpdateAuras(frame)
		else
			buffs[1]:Hide()
			buffs[1]:SetScript("OnShow", SBRF.HideMe)
			buffs[2]:Hide()
			buffs[2]:SetScript("OnShow", SBRF.HideMe)
		end
	end
end

local function DisabledAnchor(info)
	local setting = info[#info]
	return not db[anchorShowMap[setting]]
end

local function SetBuffAnchor(info, val)
	local setting = info[#info]
	db[setting] = val
	SBRF:PushUpvalues()
	local frameType = buffFrameMap[setting]
	local x, y, point, relative  = unpack(anchorData[val])
	for frame,_ in pairs(fixedFrames) do
		local buffs = frame[frameType]
		buffs[1]:ClearAllPoints()
		buffs[1]:SetPoint(val, frame, val, x, y)
		buffs[2]:ClearAllPoints()
		buffs[2]:SetPoint(point, buffs[1], relative, 0, 0)
	end
end

function SBRF:InitOptions()
	local options = {
		type = 'group',
		name = "Small Bliz Raid Frames",
		get = function(info)
			return db[info[#info]]
		end,
		args = {
			centerNames = {
				type = 'toggle',
				name = "Center Names",
				desc = "Move name to center of frame.",
				order = 20,
				set = SetCenterNames,
			},
			verticalHP = {
				type = 'toggle',
				name = "Vertical HP",
				desc = "Fill hp bottom-to-top, vertically.",
				order = 30,
				set = SetVerticalHP,
			},
			_gap_1 = { order = 100, type = "description", name = "", width = "full" },
			alpha = {
				type = 'range',
				name = "Alpha",
				desc = "Adjust transparency.",
				order = 110,
				min = 0.2,
				max = 1,
				step = 0.05,
				set = SetAlpha,
			},
			_head_buffs_ = {
				type = "header",
				name = "Buffs",
				order = 200,
			},
			showBuffs = {
				type = 'toggle',
				name = "Show",
				order = 210,
				set = SetShowBuffs,
			},
			buffAnchor = {
				type = 'select',
				name = "Anchor",
				order = 220,
				values = anchor_values,
				disabled = DisabledAnchor,
				set = SetBuffAnchor,
			},
			_head_debuffs_ = {
				type = "header",
				name = "Debuffs",
				order = 300,
			},
			showDebuffs = {
				type = 'toggle',
				name = "Show",
				order = 310,
				set = SetShowBuffs,
			},
			debuffAnchor = {
				type = 'select',
				name = "Anchor",
				order = 320,
				values = anchor_values,
				disabled = DisabledAnchor,
				set = SetBuffAnchor,
			},
			_head_dispel_debuffs_ = {
				type = "header",
				name = "Dispellable Debuffs",
				order = 400,
			},
			showDispelDebuffs = {
				type = 'toggle',
				name = "Show",
				order = 410,
				set = SetShowBuffs,
			},
			d_debuffAnchor = {
				type = 'select',
				name = "Anchor",
				order = 420,
				values = anchor_values,
				disabled = DisabledAnchor,
				set = SetBuffAnchor,
			},
		},
	}

	-- Remove any stub in the interface options, before adding the real entry
	local categories = INTERFACEOPTIONS_ADDONCATEGORIES
	for i = 1,#categories do
		if categories[i].name == "SmallBlizRaidFrames" then
			tremove(categories, i)
			break
		end
	end
	InterfaceAddOnsList_Update()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SmallBlizRaidFrames", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SmallBlizRaidFrames", "SmallBlizRaidFrames")
	SBRF.options_table = options
end
